/*
    
    1) include perlhost.h. This header defines all the
       OS Interfaces that are used by the PERL_OBJECT.
    2) define the host data structure 'CPerlHost'
    3) call the CPerlHost member function PerlCreate,
       which calls perl_alloc and perl_construct
    4) call the CPerlHost member function PerlParse
    5) call the CPerlHost member function PerlRun
    6) when done, call the CPerlHost member function PerlDestroy,
       which calls perl_destruct and perl_free
    
    Including XSUB.h brings in the files that have macros to redefine
    the perl_* functions, to work with PERL_OBJECT. Note that these 
    macros may cause problems when there are name collisions. A common 
    problem is having a local variable that has the same name as a Perl 
    function. To track down these name collisions compile using '/P' 
    option with VC++ so that you can see what the preprocessor has done.
    
*/ 
#include <EXTERN.h>
#include <perl.h>
#define NO_XSLOCKS

// Bring in the macros that redefine perl_* functions to work with
// PERL_OBJECT.
//
#include <XSUB.h>

/* for Win32 */
#include "win32iop.h"
#include <fcntl.h>
#include <perlhost.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <process.h>

// DynaLoader stuff.
//
char *staticlinkmodules[] = {
    "DynaLoader",
    NULL,
};

// More DynaLoader stuff.
//
EXTERN_C void boot_DynaLoader _((CV* cv _CPERLarg));

static void
xs_init(CPERLarg)
{
    char *file = __FILE__;
    dXSUB_SYS;
    newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
}

CPerlObj *pPerl;  // The Perl object used by the host.

int main (int argc, char **argv, char **env)
{
  struct _stat stbuf;

  char* name = argv[0];
  for (char* pp = argv[0]; *pp; pp++) {
    if (*pp == '\\' || *pp == '/') name = ++pp;
  }

  char cmd[2048] = "";

    /* Find the *real* version of this command (argv[0]) */

    char path[2048];
    strcpy (path, getenv("PATH"));
    char* path_end = path + strlen (path);
    {for (char* pp = path; pp < path_end; pp++) {
      if (*pp == ';') {*pp = 0;}
    }}

    char pathext[2048] = ";", *pPATHEXT = getenv("PATHEXT");
    strcpy (pathext+1, pPATHEXT ? pPATHEXT : ".EXE;.COM;.BAT");
    char* pathext_end = pathext + 1 + strlen (pathext+1);
    {for (char* ppe = pathext; ppe < pathext_end; ppe++) {
      if (*ppe == ';') {*ppe = 0;}
    }}

    int my_size = -1;
    {for (char* ppe = pathext; ppe < pathext_end; ppe += strlen (ppe) + 1) {
      sprintf (cmd, "%s%s", argv[0], ppe);
      if (!_stat (cmd, &stbuf)) {
        my_size = stbuf.st_size;
        break;
      }
    }}
    {for (char* pp = path; pp < path_end; pp += strlen (pp) + 1) {
      for (char* ppe = pathext; ppe < pathext_end; ppe += strlen (ppe) + 1) {
        sprintf (cmd, "%s\\%s%s", pp, name, ppe);
        if (!_stat (cmd, &stbuf)) {
          if (my_size == -1) {
            my_size = 0;
          } else if (stbuf.st_size != my_size) {
            goto FOUND;
          }
        }
      }
    }}

    // If we got here, we couldn't find the *real* perl.exe.  So just use
    // our name
    strcpy (cmd, argv[0]);

  // Now translate command line arguments to native Win32.

 FOUND:
  char* cmd_args[256], args_storage[2048], *pas = cmd + strlen(cmd);
  int iarg = 0;
  cmd_args[iarg++] = cmd;
  for (int i = 1; i < argc; i++) {
    FILE   *CYGPATH;
    char cmd[256];
    *pas++ = 0;
    cmd_args[iarg++] = pas;
    if (*argv[i] == '-') goto NOT_FILE;
    {for (char* pc = argv[i]; *pc; pc++) {
      if (*pc == ' ' || *pc == ';' || *pc == '\'' || *pc == '"') {
        goto NOT_FILE;
      }
    }}
    sprintf (cmd, "cygpath -w %s 2>nul", argv[i]);
    if( (CYGPATH = _popen(cmd, "rb" )) == NULL )
      exit( 1 );

    if ((1 == fscanf (CYGPATH, "%s", pas)) && !_stat (pas, &stbuf)) {
      pas += strlen(pas);
      _pclose(CYGPATH);
      goto NEXT_ARG;
    }
    _pclose(CYGPATH);
  NOT_FILE:
    strcpy (pas, argv[i]);
    pas += strlen(pas);
  NEXT_ARG:;
  }
  cmd_args[iarg] = 0;

  CPerlHost host;     // The Perl host
  host.PerlCreate();  // Create an instance of the Perl Interpreter

  host.PerlParse(xs_init, argc, cmd_args, NULL);

  host.PerlRun();      // Run the interpreter.
  host.PerlDestroy();  // Destroy the interpreter.
  return 0;
}
