#! /usr/local/bin/perl
#                                 SPECTRUM: PERL                         txt off
# Copyright (C) Siemens AG 1995 All Rights Reserved.Confidential
# This is a perl "program library" containing a collection of
# generally useful subroutines.
#
# $Id: tool.pl,v 1.004 1997/11/12 10:21:28 spsy Exp $
#
# $NewRev$
# $NewRevEnd$
#
# $Log: tool.pl,v $
# Revision 1.004  1997/11/12  10:21:28  spsy
# 971112 zoec: return exit status from spec_comp
#
# Revision 1.003  1995/09/29  04:02:39  ghart
# Correct test for build in progress in build directory.
#
# Revision 1.002  1995/07/17  15:23:25  ghart
# # 950717 ghart : add header
#
# $Logend$
#                                                                        txt on-

$IS_NT_PERL = defined (&Win32::IsWinNT); # If we're using NT Perl
$IS_NT = $ENV{MACHENV} eq "NT4"; # If we're on NT, different names

#****************************************************************************
sub err { # generate a message
#****************************************************************************
  ($degree, $msg) = @_;
  return if $main::SUPPRESS_ERR;
  print STDERR "$degree! $msg\n";
  if ($degree ne "WARN") {
    $errors++;
  }
  $worst =~ s/$degree.*/$degree/;
}
#****************************************************************************
sub system_err { # call system command & check status
#****************************************************************************
  ($cmd,$quiet) = @_;

  if ($VERBOSE) {
    &err ("info", "Executing: $cmd");
  }
  if ($IS_NT_PERL) {
    $cmd =~ /(\S+)/;
    $name = $1;
    if (-f &files ("$name.plx") || $name =~ /.plx$/) {
      $cmd = "perl $cmd";
    }
  } elsif ($IS_NT) {
    $cmd =~ s?\\?\\\\?g;
  }
  $stat = system ($cmd);
  if (!$quiet && $stat) {
    &err ("ERROR", "Command FAILED ($stat): $cmd");
  }
  $stat;
}
#****************************************************************************
sub debug { # generate a debug message if appropriate
#****************************************************************************
  ($key, $msg) = @_;
  if (defined ($main::DEBUG) && (index ($main::DEBUG, $key) >= 0)) {
    print STDERR "$key:$msg";
  }
}
$TRANSLATE_FILE = "";
if ($IS_NT) {
  open (PATHS, "real_path -perl|");
  while (<PATHS>) {
    m/^\s*(\S+)\s*->\s*(\S+)\s*$/;
    $TRANSLATE_FILE .= '$val =~ s?(^|\s+)' .
            (($1 eq '/') ? ('[\\/]?$1' . quotemeta($2) . '/') :
            (quotemeta ($1) . '([\\/]|$)?$1') . quotemeta($2)) .
            '$2?g;';
  }
  close (PATHS);
  $TRANSLATE_FILE .= '$val =~ tr?/?\\\\?;';
}
$sub_files = '
sub files {
  ($val) = @_;
#  print ("IN <-- $val  ");
  ' . $TRANSLATE_FILE . '
#  print ("OUT --> $val\n");
  $val;
}
';
#print "$sub_files\n";
eval $sub_files;

#****************************************************************************
sub ch_dir { # cd to directory
#****************************************************************************
  local ($dir_name) = @_;
  chdir &files ($dir_name);
}
#****************************************************************************
sub make_dir { # Create a directory with correct permissions
#****************************************************************************
  local ($dir_name) = @_;
  $dir = "";
  $dir_name = files ($dir_name);
  return 1 if (-d $dir_name);
  foreach $p (split('[/\\\\]', $dir_name)) {
    $dir .= "$p";
    if ( $dir && ! -d $dir ) {
      if (!mkdir ($IS_NT?"$dir.":$dir, 02775)) { # BUG in NT filesystem/mkdir
        &err ('FATAL', "mkdir failure ($dir): $!");
        return;
      }
      if (!$IS_NT) {
        chown (-1, (getgrnam('spec'))[2], $dir);
        # The "2_64" in 02775 above seems to be ignored
        system_err ("chmod g+s,g+w,+r $dir");
      }
    }
    $dir .= "/";
  }
  return 1;
}
#****************************************************************************
sub lock_dir { # lock directory $1 and either wait, or err out
#****************************************************************************
  # Lock out simultaneous build (on same host)

  local ($lockdir, $name, $to_wait) = @_;

  $our_pid=$$;
  if (! defined $name) {
    $name = '.LOCK';
  }
  $lock = files ("$lockdir/$name"); $tlock = "$lock.$our_pid";
  if ($IS_NT_PERL) {
    while (1) {
      if (! -f $lock) {
        if (open (LOCK, ">$lock")) {
          select (LOCK); $| = 1; select (STDOUT);
          print LOCK "$our_pid\n";
          return 1;
        }
      } else {
        if (unlink $lock) {
          if (open (LOCK, ">$lock")) {
            select (LOCK); $| = 1; select (STDOUT);
            print LOCK "$our_pid\n";
            return 1;
          }
        }
        if (open (LOCK, "<$lock")) {
          $build_pid = <LOCK>; close (LOCK);
          if ( $to_wait ne "wait" ) {
            $build_pid =~ /(\d*)/;
            &err ('FATAL', "Currently a build in $bld_dir (pid: $1)");
            return;
          }
        }
        sleep 5;
      }
    }
  }
  if (open (LOCK, "<$lock")) {
    $build_pid = <LOCK>; close (LOCK);
  } else {
    $build_pid = 0;
  }
  while ( $our_pid != $build_pid ) {
    $done = 0;
    while ( ! $done) {
      if ($IS_NT_PERL) {
print STDERR "kill -0 $build_pid 2>nul; exit \$?\n";
        $pid_stat = kill 10, $build_pid #!system ("sh","-c","'kill -0 $build_pid 2>nul; exit \$?'");
      } else {
        $pid_stat = !system ("kill -0 $build_pid 2>/dev/null");
      }
      if ($build_pid && $pid_stat) {
        if ( $to_wait ne "wait" ) {
          &err ('FATAL', "Currently a build in $bld_dir (pid: $build_pid)");
          return;
        }
        sleep 5;
      } else {
        unlink ($tlock);
        open (TLOCK, ">$tlock") || die "Can't write to lock file: $tlock\n";
        print TLOCK $our_pid; close TLOCK;
        chmod (0444, $tlock);

        # Delete the .LOCK file when we're done'
        &cleanup_file ($lock);
        unlink ($lock); rename ($tlock, $lock);
        $done = 1;
      }
    }
    sleep (0); # Try this to let any other simultaneous lock attempt in
    if (open (LOCK, "<$lock")) {
      $build_pid = <LOCK>; close (LOCK);
    } else {
      $build_pid = 0;
    }
  }
  return 1;
}
#****************************************************************************
sub unlock_dir { # dummy
#****************************************************************************
}
#****************************************************************************
sub abort { # when the script is killed, run this
#****************************************************************************
  &finished (2);
}
#****************************************************************************
sub finished { # when the script is done, run this
#****************************************************************************
  local ($status) = @_;
  unlock_dir ();
  eval $qCLEANUP;
  unlink (split( /\s+/, $qSPEC_CLEANUP_FILES));
}
# SPEC_CLEANUP_FILES # Don't reset this, it can be inherited.
#****************************************************************************
sub cleanup_file { # add $* to list of files to be deleted at exit
#****************************************************************************
  $qSPEC_CLEANUP_FILES = "$qSPEC_CLEANUP_FILES @_";
}
#****************************************************************************
sub CLEAN_UP { # This is a band-aid for AIX ksh: clean up tmp files
#****************************************************************************
# unlink (${RM} -f /tmp/sh$$.[0-9] 2>/dev/null
}

1;

